'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.run = run;

var _Subject = require('rxjs/Subject');

var _map = require('rxjs/operator/map');

function isEvent(e) {
  return e && e.preventDefault && e.stopPropagation;
}

function run(program, render) {
  var opt = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var _program$main = program.main,
      init = _program$main.init,
      view = _program$main.view,
      update = _program$main.update,
      subscriptions = _program$main.subscriptions;
  var onUpdate = opt.onUpdate;
  // Use requestAnimationFrame for better render performance

  var rafRender = function rafRender(vdom) {
    return requestAnimationFrame(function () {
      return render(vdom);
    });
  };
  var model$ = new _Subject.Subject();
  var model = init,
      initSideEffect = void 0,
      view$ = void 0;

  if (Array.isArray(model) && typeof model[1] === 'function') {
    initSideEffect = model[1];
    model = model[0];
  }

  if (typeof model === 'undefined') {
    throw new Error('Initial model is undefined. Pass null to explicitly set `no value`.');
  }

  view$ = view(dispatch, model$);
  if (typeof view$ === 'function') {
    view$ = _map.map.call(model$, view$);
  }

  view$.subscribe(rafRender);
  model$.next(model);

  initSideEffect && initSideEffect(dispatch);
  initSideEffect = null;

  subscriptions && subscriptions(dispatch, model);

  function dispatch(msg) {
    for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    if (typeof msg === 'function') {
      return function eventHandler(e) {
        if (isEvent(e)) {
          var type = e.type,
              target = e.target;

          if (type === 'change' && target.tagName === 'INPUT' && (target.type === 'radio' || target.type === 'checkbox')) {
            dispatch(new (Function.prototype.bind.apply(msg, [null].concat(args, [target.checked])))());
          } else {
            dispatch(new (Function.prototype.bind.apply(msg, [null].concat(args, [target.value])))());
          }
        } else {
          dispatch(new (Function.prototype.bind.apply(msg, [null].concat(args)))());
        }
      };
    }

    if ((typeof msg === 'undefined' ? 'undefined' : _typeof(msg)) !== 'object') {
      throw new Error('Expected msg to be an object: ' + JSON.stringify(msg));
    }

    var newModel = update(msg, model),
        sideEffect = void 0;
    if (typeof newModel === 'undefined') {
      throw new Error('Unhandled msg: ' + JSON.stringify(msg));
    }

    if (Array.isArray(newModel) && typeof newModel[1] === 'function') {
      sideEffect = newModel[1];
      newModel = newModel[0];
    }

    onUpdate && onUpdate(msg, model, newModel);

    if (newModel !== model) {
      model = newModel;
      model$.next(model);
    }

    sideEffect && sideEffect(dispatch);
  }
}